import pygame
from papi import msaa

class AccWindow (msaa.IAccessible):
    def __init__ (self):
        msaa.IAccessible.__init__ (self)
        self.children = []
        self._lastchild = 0
        
        self.accDoDefaultAction = self._accDoDefaultAction
        self.accHitTest = self._accHitTest
        self.accLocation = self._accLocation
        self.accNavigate = self._accNavigate
        self.accSelect = self._accSelect
        self.get_accName = self._get_accName
        self.get_accRole = self._get_accRole
        self.get_accChild = self._get_accChild
        self.get_accChildCount = self._get_accChildCount
        self.get_accParent = self._get_accParent
        self.get_accDefaultAction = self._get_accDefaultAction
        self.get_accDescription = self._get_accDescription
        self.get_accState = self._get_accState
        self.get_accFocus = self._get_accFocus
        self.get_accHelp = self._get_accHelp
        self.get_accHelpTopic = self._get_accHelpTopic
        self.get_accKeyboardShortcut = self._get_accKeyboardShortcut
        self.get_accSelection = self._get_accSelection
        self.get_accValue = self._get_accValue
        
    def _get_accValue (self):
        return None

    def _get_accSelection (self):
        return None

    def _get_accKeyboardShortcut (self):
        return None

    def _get_accFocus (self):
        for child in self.children:
            v = child.get_accFocus ()
            if v:
                return v
        return None

    def _get_accHelpTopic (self):
        return "Client Help Topic", 1

    def _get_accHelp (self):
        return "Client Help"

    def _accSelect (self, flags):
        return False

    def _get_accState (self):
        return None

    def _accLocation (self):
        return None

    def _get_accDescription (self):
        return "Demo Client"

    def _get_accDefaultAction (self):
        return "Client Action!"

    def _accDoDefaultAction (self):
        return None

    def _get_accParent (self):
        return None

    def _accHitTest (self, x, y):
        for child in self.children:
            v = child.accHitTest (x, y)
            if v:
                return v
        return self

    def _accNavigate (self, x):
        if len (self.children) == 0:
            return None
        if x == msaa.NAVDIR_FIRSTCHILD:
            self._lastchild = 0
            return self.children[0]
        elif x == msaa.NAVDIR_LASTCHILD:
            self._lastchild = len (self.children) - 1
            return self.children[-1]
        return None

    def _get_accName (self):
        return "Example Client"
        
    def _get_accChild (self, val):
        if len (self.children) > 0:
            return self.children[val - 1]

    def _get_accChildCount (self):
        return len (self.children)

    def _get_accRole (self):
        return msaa.ROLE_SYSTEM_CLIENT

class AccButton (msaa.IAccessible):
    def __init__ (self):
        msaa.IAccessible.__init__ (self)
        self._pushcount = 0
        self.parent = None
        self._focused = False
        
        self.accDoDefaultAction = self._accDoDefaultAction
        self.accHitTest = self._accHitTest
        self.accLocation = self._accLocation
        self.accNavigate = self._accNavigate
        self.accSelect = self._accSelect
        self.get_accName = self._get_accName
        self.get_accRole = self._get_accRole
        self.get_accChild = self._get_accChild
        self.get_accChildCount = self._get_accChildCount
        self.get_accParent = self._get_accParent
        self.get_accDefaultAction = self._get_accDefaultAction
        self.get_accDescription = self._get_accDescription
        self.get_accState = self._get_accState
        self.get_accFocus = self._get_accFocus
        self.get_accHelp = self._get_accHelp
        self.get_accHelpTopic = self._get_accHelpTopic
        self.get_accKeyboardShortcut = self._get_accKeyboardShortcut
        self.get_accSelection = self._get_accSelection
        self.get_accValue = self._get_accValue

    def _get_accValue (self):
        return str (self._pushcount)

    def _get_accSelection (self):
        return None

    def _get_accKeyboardShortcut (self):
        return "Strg+B"

    def _get_accFocus (self):
        if self._focused:
            return self
        return None

    def _get_accHelpTopic (self):
        return "Button Help Topic", 1

    def _get_accHelp (self):
        return "Button Help"

    def _accSelect (self, flags):
        if flags & msaa.SELFLAG_NONE == msaa.SELFLAG_NONE:
            return True
        if flags & msaa.SELFLAG_TAKEFOCUS == msaa.SELFLAG_TAKEFOCUS:
            self._focused = True
            return True
        return False

    def _get_accState (self):
        return msaa.STATE_SYSTEM_DEFAULT

    def _accLocation (self):
        return 10, 10, 50, 50

    def _get_accDescription (self):
        return "Simple Button"

    def _get_accDefaultAction (self):
        return "Button press"

    def _accDoDefaultAction (self):
        self._pushcount += 1

    def _get_accParent (self):
        return self.parent

    def _accHitTest (self, x, y):
        if 10 <= x < 60 and 10 <= y < 60:
            return self

    def _accNavigate (self, x):
        return None

    def _get_accName (self):
        return "Example Button"
        
    def _get_accChild (self, val):
        return None

    def _get_accChildCount (self):
        return 0

    def _get_accRole (self):
        return msaa.ROLE_SYSTEM_PUSHBUTTON

if __name__ == "__main__":
    pygame.init ()
    screen = pygame.display.set_mode ((300, 300))
    msaa.hook_on(pygame.display.get_wm_info ()["window"])
    acc = AccWindow ()
    msaa.set_root (acc)
    button = AccButton ()
    acc.children.append (button)
    button.parent = acc

    istrue = True
    while istrue:
        for e in pygame.event.get ():
            if e.type == pygame.QUIT:
                istrue = False
