#
# Copyright (c) 2008, Marcus von Appen
# All rights reserved.
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  * Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import papi.msaa as msaa

class MSAAAccessible (object):
    """Accessible () -> Accessible
    
    """
    def __init__ (self, acc=None):
        self._name = None
        self._description = None
        self._children = {}
        self._parent = None
        self._role = None

        if acc:
            self._acc = acc
        else:
            self._bind_internals ()

    def bind_internals (self):
        self._acc = msaa.IAccessible ()
        self._acc.get_accName = self.get_name
        self._acc.get_accDescritpion = self.get_description
        self._acc.get_accParent = self.get_parent
        self._acc.get_accRole = self.get_role
        self._acc.get_accChildCount = self.get_childcount
        self._acc.get_accChild = self._get_child
        
        self._acc.accDoDefaultAction = self._do_nada
        self._acc.accHitTest = self._do_nada
        self._acc.accLocation = self._do_nada
        self._acc.accNavigate = self._do_nada
        self._acc.accSelect = self._do_nada
        self._acc.get_accDefaultAction = self._do_nada
        self._acc.get_accFocus = self._do_nada
        self._acc.get_accHelp = self._do_nada
        self._acc.get_accHelpTopic = self._do_nada
        self._acc.get_accKeyboardShortcut = self._do_nada
        self._acc.get_accSelection = self._do_nada
        self._acc.get_accValue = self._do_nada
        
    def _do_nada (self, *args):
        return None

    def _get_child (self, id):
        acc = self.get_child (id)
        if acc:
            return acc._acc
        return None

    def get_name (self):
        """A.get_name () -> str
        
        Gets the name of the Accessible.
        """
        return self._name
    
    def set_name (self, name):
        """A.set_name (name) -> None
        
        Sets the name of the Accessible.
        """
        self._name = name

    def get_description (self):
        """A.get_description () -> str
        
        Gets the description of the Accessible.
        """
        return self._description

    def set_description (self, description):
        """A.set_description (description) -> None
        
        Sets the description of the Accessible.
        """
        self._description = description
        
    def get_parent (self):
        """A.get_parent () -> str
        
        Gets the parent of the Accessible.
        """
        return self._parent

    def set_parent (self, parent):
        """A.set_parent (parent) -> None
        
        Sets the parent of the Accessible.
        """
        self._parent = parent
        
    def get_child (self, id):
        """A.get_child (id) -> Accessible
        
        Gets the child with the matching id from the Accessible.
        """
        if self._children.has_key (id):
            return self._children[id]
        return None

    def get_childcount (self):
        """A.get_childcount () -> int
        
        Gets the number of children for the Accessible.
        """
        return len (self._children)

    def get_role (self):
        """A.get_role () -> int
        
        Gets the role of the Accessible.
        """
        return self._role

    def set_role (self, role):
        """A.set_role (role) -> None
        
        Sets the role of the Accessible.
        """
        self._role = role
