/*
 * Copyright (c) 2008, Marcus von Appen
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#define PAPI_IACCESSIBLE_INTERNAL

#include "papi_msaa.h"
#include "papi_msaaprivate.h"
#include "papi_iaccproxy.h"

/* Forward declarations of the PyIAccessible bindings. */
static PyObject* _not_implemented_args (PyObject *self, PyObject *args);
static PyObject* _not_implemented_noargs (PyObject *self);
static PyObject* _iaccessible_new (PyTypeObject *type, PyObject *args,
                                   PyObject *kwds);
static int _iaccessible_init (PyIAccessible *self, PyObject *args,
                              PyObject *kwds);
static void _iaccessible_dealloc (PyIAccessible *self);
static PyObject* _iaccessible_get_dict (PyIAccessible *self, void *closure);

/**
 * Methods, which are bound to the PyIAccessible type.
 */
static PyMethodDef _iaccessible_methods[] =
{
    { "accDoDefaultAction", (PyCFunction)_not_implemented_noargs, METH_NOARGS,
      "O.accDoDefaultAction () -> None\n\n"
      "Executes the object's default action, if any." },
    { "accHitTest", _not_implemented_args, METH_VARARGS,
      "O.accHitTest (left, top) -> IAccessible\n\n"
      "Retrieves the child object at the given coordinates on the screen,\n"
      "left and top specify the point on the screen to check for a child.\n"
      "If no IAccessible child can be found, None is returned." },
    { "accLocation", (PyCFunction)_not_implemented_noargs, METH_NOARGS,
      "O.accLocation () -> (left, top, right, bottom)\n\n"
      "Retrieves the object's current screen location.\n\n"
      "Retrieves the object's screen location. If the object is the main\n"
      "root object, this method can be left unimplemented or return None\n"
      "to automatically return its window's location." },
    { "accNavigate", _not_implemented_args, METH_VARARGS,
      "O.accNavigate (direction) -> IAccessible\n\n"
      "Navigates to the next child or sibling IAccessible according to the\n"
      "passed direction argument. The direction is one of the NAVDIR*\n"
      "constants.\n\n"
      "If no child or sibling can be found, None is returned." },
    { "accSelect", _not_implemented_args, METH_VARARGS,
      "O.accSelect (flags) -> bool\n\n"
      "Modifies the selection or moves the keyboard focus of the object.\n\n"
      "Modifies the selection or moves the keyboard focus of the object\n"
      "according to the passed flags argument. flags is one of the SELFLAGS*\n"
      "constants." },
    { "get_accChild", (PyCFunction) _not_implemented_args, METH_VARARGS,
      "O.get_accChild (id) -> IAccessible\n\n"
      "Retrieves the child with the specified id for this IAccessible" },
    { "get_accChildCount", (PyCFunction) _not_implemented_noargs, METH_NOARGS,
      "O.get_accChildCount () -> int\n\n"
      "Retrieves the number of children that belong to this IAccessible." },
    { "get_accDefaultAction", (PyCFunction) _not_implemented_noargs,
      METH_NOARGS,
      "O.get_accDefaultAction () -> str\n\n"
      "Gets a string that describes the default action of the IAccessible.\n\n"
      "If the IAccessible does not support a default action, None will be\n"
      "returned." },
    { "get_accDescription", (PyCFunction) _not_implemented_noargs, METH_NOARGS,
      "O.get_accDescription () -> str\n\n"
      "Gets a string that describes the visual appearance.\n\n"
      "In case the object does not need any description, None will be\n"
      "returned." },
    { "get_accFocus", (PyCFunction) _not_implemented_noargs, METH_NOARGS,
      "O.get_accFocus () -> IAccessible\n\n"
      "Retrieves the IAccessible that has the keyboard focus." },
    { "get_accHelp", (PyCFunction) _not_implemented_noargs, METH_NOARGS,
      "O.get_accHelp () -> str\n\n"
      "Retrieves the IAccessible's help property string.\n\n"
      "If the IAccessible does not have any help, None will be returned." },
    { "get_accHelpTopic", _not_implemented_args, METH_VARARGS,
      "O.get_accHelpTopic () -> (str, int)\n\n"
      "Gets the WinHelp file and index within it for this IAccessible.\n\n"
      "Retrieves the full path of the WinHelp file for the IAccessible and\n"
      "the identifier of the topic desribing it.\n"
      "If the IAccessible does not have any help, None will be returned." },
    { "get_accKeyboardShortcut", (PyCFunction) _not_implemented_noargs,
      METH_NOARGS,
      "O.get_accKeyboardShortcut () -> str\n\n"
      "Gets the mnemonic key for the IAccessible." },
    { "get_accName", (PyCFunction) _not_implemented_noargs, METH_NOARGS,
      "O.get_accName () -> str\n\n"
      "Gets the name of the IAccessible."},
    { "get_accParent", (PyCFunction) _not_implemented_noargs, METH_NOARGS,
      "O.get_accParent () -> IAccessible\n\n"
      "Gets the parent IAccessible for this object.\n\n"
      "Returns the parent IAccessible for this object or None, if it does\n"
      "not have any." },
    { "get_accRole", (PyCFunction) _not_implemented_noargs, METH_NOARGS,
      "O.get_accRole () -> int\n\n"
      "Retrieves the role of the IAccessible.\n\n"
      "Returns the role of the IAccessible. The IAccessible's role is a\n"
      "valid ROLE_* constant." },
    { "get_accSelection", (PyCFunction) _not_implemented_noargs, METH_NOARGS,
      "O.get_accSelection -> list\n\n"
      "Gets a list of the currently selected children for the IAccessible.\n\n"
      "If the IAccessible does not have any selected children, None will be\n"
      "returned." },
    { "get_accState", (PyCFunction) _not_implemented_noargs, METH_NOARGS,
      "O.get_accState () -> int\n\n"
      "Gets the current state of the IAccessible.\n\n"
      "Returns the current state of the IAccessible. The IAccessible's state\n"
      "is a valid STATE_* constant." },
    { "get_accValue", (PyCFunction) _not_implemented_noargs, METH_NOARGS,
      "Gets the value of the IAccessible.\n\n"
      "Returns the value of the IAccessible or None, in case it does not\n"
      "have or support any value." },
    { NULL, NULL, 0, NULL }

//     { "put_accName", _not_implemented_args, METH_VARARGS, NULL },
//     { "put_accValue", _not_implemented_args, METH_VARARGS, NULL },
};

/**
 * Getters and setters for the PyIAccessible.
 */
static PyGetSetDef _iaccessible_getsets[] = {
    { "__dict__", (getter) _iaccessible_get_dict, NULL, NULL, NULL },
    { NULL, NULL, NULL, NULL, NULL }
};


PyTypeObject PyIAccessible_Type =
{
    PyObject_HEAD_INIT(NULL)
    0,
    "papi.msaa.IAccessible",    /* tp_name */
    sizeof (PyIAccessible),     /* tp_basicsize */
    0,                          /* tp_itemsize */
    (destructor) _iaccessible_dealloc, /* tp_dealloc */
    0,                          /* tp_print */
    0,                          /* tp_getattr */
    0,                          /* tp_setattr */
    0,                          /* tp_compare */
    0,                          /* tp_repr */
    0,                          /* tp_as_number */
    0,                          /* tp_as_sequence */
    0,                          /* tp_as_mapping */
    0,                          /* tp_hash */
    0,                          /* tp_call */
    0,                          /* tp_str */
    0,                          /* tp_getattro */
    0,                          /* tp_setattro */
    0,                          /* tp_as_buffer */
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /* tp_flags */
    "IAccessible () -> IAccessible\n\n"
    "MSAA accessibility object.\n\n"
    "For a detailed documentation about its methods, visit\n"
    "http://msdn.microsoft.com/en-us/library/ms696165(VS.85).aspx",
    0,                          /* tp_traverse */
    0,                          /* tp_clear */
    0,                          /* tp_richcompare */
    offsetof (PyIAccessible, weakrefs), /* tp_weaklistoffset */
    0,                          /* tp_iter */
    0,                          /* tp_iternext */
    _iaccessible_methods,         /* tp_methods */
    0,                          /* tp_members */
    _iaccessible_getsets,         /* tp_getset */
    0,                          /* tp_base */
    0,                          /* tp_dict */
    0,                          /* tp_descr_get */
    0,                          /* tp_descr_set */
    offsetof (PyIAccessible, dict), /* tp_dictoffset */
    (initproc) _iaccessible_init, /* tp_init */
    0,                          /* tp_alloc */
    _iaccessible_new,           /* tp_new */
    0,                          /* tp_free */
    0,                          /* tp_is_gc */
    0,                          /* tp_bases */
    0,                          /* tp_mro */
    0,                          /* tp_cache */
    0,                          /* tp_subclasses */
    0,                          /* tp_weaklist */
    0                           /* tp_del */
};

#define PyIAccessible_Check(x) ((x)->ob_type == &PyIAccessible_Type)

/**
 * Creates a new PyIAccessible.
 */
static PyObject*
_iaccessible_new (PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    debug ("_iaccessible_new\n");
    return PyType_GenericNew (type, args, kwds);
}

/**
 * Initializes the PyIAccessible members, such as the IAccProxy.
 */
static int
_iaccessible_init (PyIAccessible *self, PyObject *args, PyObject *kwds)
{
    debug ("_iaccessible_init\n");
    if (!self->obj)
        self->obj = new IAccProxy ((PyObject*) self);
    self->obj->hwnd = get_root_hwnd ();
    return 0;
}

/**
 * Deallocates the PyIAccessible and its members.
 */
static void
_iaccessible_dealloc (PyIAccessible *self)
{
    debug ("_iaccessible_dealloc\n");

    if (self->obj != NULL)
    {
        self->obj->ResetPythonRef ();
        /* Do not use delete - maybe it's hold by some
         * MSAA client.
         */
        self->obj->Release ();
    }

    Py_XDECREF (self->dict);
    Py_XDECREF (self->prop_handlers);

    if (self->weakrefs)
        PyObject_ClearWeakRefs ((PyObject *) self);

    self->ob_type->tp_free ((PyObject *) self);
}

/**
 * Getter for IAccessible.__dict__
 */
static PyObject*
_iaccessible_get_dict (PyIAccessible *self, void *closure)
{
    if (!self->dict)
    {
	    self->dict = PyDict_New ();
        if (!self->dict)
            return NULL;
    }

    Py_INCREF (self->dict);
    return self->dict;
}

/**
 * Default implementation for the interfaces.
 */
static PyObject*
_not_implemented_noargs (PyObject *self)
{
    PyErr_SetString (PyExc_NotImplementedError, "method not implemented");
    return NULL;
}

/**
 * Default implementation for the interfaces.
 */
static PyObject*
_not_implemented_args (PyObject *self, PyObject *args)
{
    PyErr_SetString (PyExc_NotImplementedError, "method not implemented");
    return NULL;
}

static PyObject*
PyIAccessible_New ()
{
    PyIAccessible *self = (PyIAccessible *) PyType_GenericNew
        (&PyIAccessible_Type, NULL, NULL);
    self->obj = new IAccProxy ((PyObject*) self);
    return (PyObject *) self;
}

void
iaccessible_export_api (void *c_api[])
{
    c_api[PAPI_IACCESSIBLE_FIRSTSLOT] = &PyIAccessible_Type;
    c_api[PAPI_IACCESSIBLE_FIRSTSLOT + 1] = (void*) &PyIAccessible_New;
}
